@@ -29,7 +29,7 @@ use super::states::{
     ServiceStatesProbeNodeReplicaMetricsRabbitMQ,
 };
 use super::status::Status;
-use crate::config::config::ConfigPluginsRabbitMQ;
+use crate::config::config::{ConfigPluginsRabbitMQ, ConfigProbeServiceNodeHTTPMethod};
 use crate::config::regex::Regex;
 use crate::prober::manager::STORE as PROBER_STORE;
 use crate::prober::mode::Mode;
@@ -68,7 +68,13 @@ pub struct Store {
 }
 
 enum DispatchMode<'a> {
-    Poll(&'a ReplicaURL, &'a Option<Regex>),
+    Poll(
+        &'a ReplicaURL,
+        &'a HeaderMap,
+        &'a Option<ConfigProbeServiceNodeHTTPMethod>,
+        &'a Option<String>,
+        &'a Option<Regex>,
+    ),
     Script(&'a String),
 }
 
@@ -85,7 +91,16 @@ fn make_default_headers() -> HeaderMap {
     headers
 }
 
-fn map_poll_replicas() -> Vec<(String, String, String, ReplicaURL, Option<Regex>)> {
+fn map_poll_replicas() -> Vec<(
+    String,
+    String,
+    String,
+    ReplicaURL,
+    HeaderMap,
+    Option<ConfigProbeServiceNodeHTTPMethod>,
+    Option<String>,
+    Option<Regex>,
+)> {
     let mut replica_list = Vec::new();
 
     // Acquire states
@@ -106,6 +121,9 @@ fn map_poll_replicas() -> Vec<(String, String, String, ReplicaURL, Option<Regex>
                             node_id.to_owned(),
                             replica_id.to_owned(),
                             replica_url.to_owned(),
+                            node.http_headers.to_owned(),
+                            node.http_method.to_owned(),
+                            node.http_body.to_owned(),
                             node.http_body_healthy_match.to_owned(),
                         ));
                     }
@@ -148,6 +166,9 @@ fn map_script_replicas() -> Vec<(String, String, String, String)> {
 
 fn proceed_replica_probe_poll_with_retry(
     replica_url: &ReplicaURL,
+    http_headers: &HeaderMap,
+    http_method: &Option<ConfigProbeServiceNodeHTTPMethod>,
+    http_body: &Option<String>,
     body_match: &Option<Regex>,
 ) -> (Status, Option<Duration>) {
     let (mut status, mut latency, mut retry_count) = (Status::Dead, None, 0);
@@ -162,7 +183,13 @@ fn proceed_replica_probe_poll_with_retry(
 
         thread::sleep(Duration::from_millis(PROBE_HOLD_MILLISECONDS));
 
-        let probe_results = proceed_replica_probe_poll(replica_url, body_match);
+        let probe_results = proceed_replica_probe_poll(
+            replica_url,
+            http_headers,
+            http_method,
+            http_body,
+            body_match,
+        );
 
         status = probe_results.0;
         latency = Some(probe_results.1);
@@ -173,6 +200,9 @@ fn proceed_replica_probe_poll_with_retry(
 
 fn proceed_replica_probe_poll(
     replica_url: &ReplicaURL,
+    http_headers: &HeaderMap,
+    http_method: &Option<ConfigProbeServiceNodeHTTPMethod>,
+    http_body: &Option<String>,
     body_match: &Option<Regex>,
 ) -> (Status, Duration) {
     let start_time = SystemTime::now();
@@ -180,8 +210,9 @@ fn proceed_replica_probe_poll(
     let (is_up, poll_duration) = match replica_url {
         &ReplicaURL::ICMP(ref host) => proceed_replica_probe_poll_icmp(host),
         &ReplicaURL::TCP(ref host, port) => proceed_replica_probe_poll_tcp(host, port),
-        &ReplicaURL::HTTP(ref url) => proceed_replica_probe_poll_http(url, body_match),
-        &ReplicaURL::HTTPS(ref url) => proceed_replica_probe_poll_http(url, body_match),
+        &ReplicaURL::HTTP(ref url) | &ReplicaURL::HTTPS(ref url) => {
+            proceed_replica_probe_poll_http(url, http_headers, http_method, http_body, body_match)
+        }
     };
 
     let duration_latency = match poll_duration {
@@ -357,6 +388,9 @@ fn proceed_replica_probe_poll_tcp(host: &str, port: u16) -> (bool, Option<Durati
 
 fn proceed_replica_probe_poll_http(
     url: &str,
+    http_headers: &HeaderMap,
+    http_method: &Option<ConfigProbeServiceNodeHTTPMethod>,
+    http_body: &Option<String>,
     body_match: &Option<Regex>,
 ) -> (bool, Option<Duration>) {
     // Acquire query string separator (if the URL already contains a query string, use append mode)
@@ -370,56 +404,95 @@ fn proceed_replica_probe_poll_http(
         time::now().to_timespec().sec
     );
 
-    debug!("prober poll will fire for http target: {}", &url_bang);
-
-    let response = if body_match.is_some() {
-        PROBE_HTTP_CLIENT.get(&url_bang).send()
+    // Acquire effective HTTP method to use for probe query
+    let effective_http_method = http_method.as_ref().unwrap_or(if body_match.is_some() {
+        &ConfigProbeServiceNodeHTTPMethod::Get
     } else {
-        PROBE_HTTP_CLIENT.head(&url_bang).send()
-    };
+        &ConfigProbeServiceNodeHTTPMethod::Head
+    });
 
-    if let Ok(response_inner) = response {
-        let status_code = response_inner.status().as_u16();
+    // Acquire effective HTTP body to use for probe query (for POST methods only)
+    let effective_http_body = http_body.as_ref().map(String::as_str).unwrap_or_default();
 
-        debug!(
-            "prober poll result received for http target: {} with status: {}",
-            &url_bang, status_code
-        );
+    // Probe target, with provided HTTP method and body (if any)
+    debug!(
+        "prober poll will fire for http target: {} with method: {:?} and body: '{}'",
+        &url_bang, &effective_http_method, &effective_http_body
+    );
 
-        // Consider as UP?
-        if status_code >= APP_CONF.metrics.poll_http_status_healthy_above
-            && status_code < APP_CONF.metrics.poll_http_status_healthy_below
-        {
-            // Check response body for match? (if configured)
-            if let &Some(ref body_match_regex) = body_match {
-                if let Ok(text) = response_inner.text() {
-                    debug!(
+    let response = match effective_http_method {
+        ConfigProbeServiceNodeHTTPMethod::Head => PROBE_HTTP_CLIENT.head(&url_bang),
+        ConfigProbeServiceNodeHTTPMethod::Get => PROBE_HTTP_CLIENT.get(&url_bang),
+        ConfigProbeServiceNodeHTTPMethod::Post => {
+            PROBE_HTTP_CLIENT
+                .post(&url_bang)
+                .body(reqwest::blocking::Body::from(
+                    effective_http_body.to_string(),
+                ))
+        }
+        ConfigProbeServiceNodeHTTPMethod::Put => {
+            PROBE_HTTP_CLIENT
+                .put(&url_bang)
+                .body(reqwest::blocking::Body::from(
+                    effective_http_body.to_string(),
+                ))
+        }
+        ConfigProbeServiceNodeHTTPMethod::Patch => {
+            PROBE_HTTP_CLIENT
+                .patch(&url_bang)
+                .body(reqwest::blocking::Body::from(
+                    effective_http_body.to_string(),
+                ))
+        }
+    }
+    .headers(http_headers.to_owned())
+    .send();
+
+    match response {
+        Ok(response_inner) => {
+            let status_code = response_inner.status().as_u16();
+
+            debug!(
+                "prober poll result received for http target: {} with status: {}",
+                &url_bang, status_code
+            );
+
+            // Consider as UP?
+            if status_code >= APP_CONF.metrics.poll_http_status_healthy_above
+                && status_code < APP_CONF.metrics.poll_http_status_healthy_below
+            {
+                // Check response body for match? (if configured)
+                if let &Some(ref body_match_regex) = body_match {
+                    if let Ok(text) = response_inner.text() {
+                        debug!(
                         "checking prober poll response text for http target: {} for any match: {}",
                         &url_bang, &text
                     );
 
-                    // Doesnt match? Consider as DOWN.
-                    if body_match_regex.is_match(&text) == false {
+                        // Doesnt match? Consider as DOWN.
+                        if body_match_regex.is_match(&text) == false {
+                            return (false, None);
+                        }
+                    } else {
+                        debug!(
+                            "could not unpack response text for http target: {}",
+                            &url_bang
+                        );
+
+                        // Consider as DOWN (the response text could not be checked)
                         return (false, None);
                     }
-                } else {
-                    debug!(
-                        "could not unpack response text for http target: {}",
-                        &url_bang
-                    );
-
-                    // Consider as DOWN (the response text could not be checked)
-                    return (false, None);
                 }
-            }
 
-            return (true, None);
+                return (true, None);
+            }
+        }
+        Err(err) => {
+            debug!(
+                "prober poll result was not received for http target: {} (error: {})",
+                &url_bang, err
+            );
         }
-    } else {
-        debug!(
-            "prober poll result was not received for http target: {}",
-            &url_bang
-        );
     }
 
     // Consider as DOWN.
@@ -550,8 +623,14 @@ fn proceed_rabbitmq_queue_probe(
 fn dispatch_replica<'a>(mode: DispatchMode<'a>, probe_id: &str, node_id: &str, replica_id: &str) {
     // Acquire replica status (with optional latency)
     let (replica_status, replica_latency) = match mode {
-        DispatchMode::Poll(replica_url, body_match) => {
-            proceed_replica_probe_poll_with_retry(replica_url, body_match)
+        DispatchMode::Poll(replica_url, http_headers, http_method, http_body, body_match) => {
+            proceed_replica_probe_poll_with_retry(
+                replica_url,
+                http_headers,
+                http_method,
+                http_body,
+                body_match,
+            )
         }
         DispatchMode::Script(script) => proceed_replica_probe_script(script),
     };
@@ -582,7 +661,13 @@ fn dispatch_polls() {
     // Probe hosts
     for probe_replica in map_poll_replicas() {
         dispatch_replica(
-            DispatchMode::Poll(&probe_replica.3, &probe_replica.4),
+            DispatchMode::Poll(
+                &probe_replica.3,
+                &probe_replica.4,
+                &probe_replica.5,
+                &probe_replica.6,
+                &probe_replica.7,
+            ),
             &probe_replica.0,
             &probe_replica.1,
             &probe_replica.2,
@@ -714,6 +799,9 @@ pub fn initialize_store() {
                 label: node.label.to_owned(),
                 mode: node.mode.to_owned(),
                 replicas: IndexMap::new(),
+                http_headers: node.http_headers.to_owned(),
+                http_method: node.http_method.to_owned(),
+                http_body: node.http_body.to_owned(),
                 http_body_healthy_match: node.http_body_healthy_match.to_owned(),
                 rabbitmq: node.rabbitmq_queue.as_ref().map(|queue| {
                     ServiceStatesProbeNodeRabbitMQ {
